#!/usr/bin/env python

import inc
import os
import sys
import database 
import pcaputil
import ConfigParser

traceid = -1

cfg_net ="network"
cfg_intnet = "internal_net"
cfgfile="./conf/properties.cfg"

"""
 simple list of internal networks -- used for determining
 ext ip in 
 
"""
dbObj = None

internal_networks = []

show_warnings = """show warnings"""

fileflagquery ="""SELECT flags FROM tracefile WHERE id = %s"""

bulkloadquery ="""LOAD DATA LOCAL INFILE %s 
        INTO TABLE packets 
        FIELDS TERMINATED BY '|' 
        IGNORE 3 LINES"""

updateflagquery = """UPDATE tracefile SET flags = '2' WHERE id = %s"""

insert_ext_ip = """insert into external_ip (ip) 
select distinct(extip) from sockets
where
idtrace = %s and extip not in ("""

class Loader:
    def __init__(self, config=cfgfile):
        self.extractloc = None
        self.config = self.loadConfig(config)
        self.traceid = -1
        self.dbObj=database.DbUtil(config)

    def fileExists(self, filename):

        """ returns a tuple -- hexdigest and filesize"""
        (md5hash, filesize) = pcaputil.getMd5Hash(filename)
        sHash = "%32s" % md5hash
        findquery = """SELECT id from tracefile where uid = %s"""

        dbObj = self.dbObj

        conn = dbObj.getConn()
        id = dbObj.singleItemQuery(conn, findquery, (sHash))

        if not id is None: # we've seen it
            pcaputil.perror("File exists")
            return True

        insquery = """INSERT INTO tracefile (uid, filename, dtcreated, size) VALUES (%s, %s, NOW(), %s)"""

        self.traceid = dbObj.runInsert(conn, insquery, (sHash, filename, filesize))

        dbObj.returnConn(conn)

        return False


    def loadConfig(self, configfile):
        global internal_networks

        config = ConfigParser.ConfigParser()

        try:
            config.readfp(open(configfile))
        except IOError:
            pcaputil.perror( "Couldn't open %s" % configfile)
            sys.exit(2) 


        if config.has_section('data'):
            try:
                self.extractloc = config.get('data', 'extractdir')        
                if len(self.extractloc) == 0:
                    pcaputil.perror( "no extract location set")

            except ConfigParser.NoOptionError:
                pcapUtil.perror( "Illegal configuration -- network block")
                sys.exit(1)

        if config.has_section(cfg_net):
            # return all items in section
            try:
                intnetlist=config.get(cfg_net, cfg_intnet)
                if len(intnetlist) == 0:
                    pcaputil.perror( "illegal list")
                    sys.exit(1)

                l = intnetlist.split(',')

                for item in l:
                    internal_networks.append(item.strip())

            except ConfigParser.NoOptionError:
                pcapUtil.perror( "Illegal configuration -- network block")
                sys.exit(1)

        else:
            pcaputil.perror("Illegal configuration\n")
            sys.exit(1)

        return config

    def processFile(self, filename):
        dbObj = self.dbObj

        dbconn = dbObj.getConn()
        
        """ first confirm that the file has been properly flagged
            in the database
        """
        dfile = open(filename, "r")

        # pull off the header of the file first
        traceid = dfile.readline()
        #print traceid
        if traceid is None:
            pcaputil.perror( "invalid file:%s -- no traceid" % filename)
            self.traceid = -1
            return 
        else:
            self.traceid = traceid[traceid.find(":")+1:]

        hash = dfile.readline()
        if hash is None:
            pcaputil.perror("invalid file: %s -- no hash" % filename)
            return 
        else:
            hash = hash[hash.find(":")+1:]

        name = dfile.readline()
        if name is None:
            pcaputil.perror( "invalid file: %s -- bad filename" % filename)
            return 
        else:
            name = name[name.find(":")+1:]

        size = dfile.readline()
        if size is None:
            pcaputil.perror("invalid file -- zero length.  %s" % filename)
            return 
        else:
            size = size[size.find(":")+1:]

        dbflag = dbObj.singleItemQuery(dbconn, fileflagquery, (self.traceid.strip()))


        if dbflag is None:
            pcaputil.perror("Error -- no file found")
            return 

        dfile.close()


        if int(dbflag[0]) != 1:
            pcaputil.perror( "bad file flag state (%s) for file %s" % (dbflag[0], filename))
            return;

        """ and now we're orf!"""
        blquery = bulkloadquery
        dbObj.singleItemQuery(dbconn, blquery, (os.path.abspath(filename)))
     
        dbObj.returnConn(dbconn)
        dbObj.cleanup()

        dbconn = dbObj.getConn()


        #print "query %s\nid: %s" % (updateflagquery, self.traceid)

        """ update the db flag in tracefile""" 
        dbObj.singleItemQuery(dbconn, updateflagquery, (self.traceid))

        dbObj.returnConn(dbconn)
        dbObj.cleanup()


    def process(self):

        conn = self.dbObj.getConn()

        """ crack open a directory and handle each file """
        if os.path.isfile(self.extractloc):
            print "processing %s" % self.extractloc
            self.processFile(self.extractloc)
        else:
            for file in os.listdir(self.extractloc):
                path = os.path.join(self.extractloc, file)
                if os.path.isfile(path):
                    print "processing %s" % path
                    self.processFile(path)
                else:
                    """ skip for now -- else we get all recursive """
                    print "skipping subdir %s" % path

        self.dbObj.returnConn(conn)
        self.dbObj.cleanup()

if __name__ == "__main__":

    if len(sys.argv) < 2:
        print "usage: %s <dir of dump files>" % sys.argv[0]
        print "usage: %s <filename>" % sys.argv[0]
        sys.exit(1)
    elif len(sys.argv) == 3:
        dumpdir = sys.argv[1]
        cfgfile=sys.argv[2]
    else:
        dumpdir = sys.argv[1]
        cfgfile="./conf/properties.cfg"

    myloader = Loader(dumpdir, cfgfile)
    myloader.process()

