#!/usr/bin/env python

import inc
import os
import sys
import pcaputil
import database 
import ConfigParser

dbObj = None
gcfgfile="./conf/properties.cfg"
select_trace = """select id from tracefile where flags = 2"""

update_outbytes ="""update sockets set outbytes =
    (
        select sum(packetsize)
        from packets
        where idtrace = %s
        and  srcip not in (select extip from (select * from sockets) as stemp)
        and packets.idsocket = sockets.idsock
    ) where idtrace = %s"""


update_inbytes ="""update sockets set inbytes =
    (   
        select sum(packetsize)
        from packets
        where idtrace = %s
        and  srcip in (select extip from (select * from sockets) as stemp)
        and packets.idsocket = sockets.idsock
    ) where idtrace = %s"""

"""
    to workaround a current restriction  in the version of mysql I'm using (5.0)
    which precludes updating on select from the same table -- we have to use a funky
    query.

    Explanation from docs is that mysql creates temp tables for subqueries in from clause.
    So, we just wrap that in ANOTHER subquery
"""
update_duration="""update sockets set duration=(      
    select lastpacket - firstpacket from (
        select  * from sockets) as stemp 
    where idtrace = %s      
    and stemp.idsock = sockets.idsock ) 
where idtrace =%s 
"""

update_lastpack="""update sockets set lastpacket =     
    (
        select max(timestamp)
        from packets
        where idtrace = %s
        and packets.idsocket = sockets.idsock    
    ) where idtrace = %s"""

update_firstpack="""update sockets set firstpacket =     
    (
        select min(timestamp)
        from packets
        where idtrace = %s
        and packets.idsocket = sockets.idsock    
    ) where idtrace = %s"""

up_flag_full="""update sockets set dataflag = 1
where idsock in (select distinct(idsocket) from packets  where idsocket  IN ( select distinct(idsocket) from packets where flags = "SYN" and idtrace =%s) AND  ( idsocket IN  (select distinct(idsocket) from packets where flags = "FIN,ACK" and idtrace = %s)    OR  idsocket IN ( select distinct(idsocket) from packets where flags = "RST" and idtrace = %s) ))"""

up_flag_finack="""update sockets set dataflag = 2 where idsock in (select distinct(idsocket) from packets  where idsocket NOT IN ( select distinct(idsocket) from packets where flags = "SYN" and idtrace =%s) AND  idsocket IN  (select distinct(idsocket) from packets where flags = "FIN,ACK" and idtrace = %s) )"""

up_flag_rst="""update sockets set dataflag = 4 where idsock in (select distinct(idsocket) from packets  where idsocket NOT IN ( select distinct(idsocket) from packets where flags = "SYN" and idtrace = %s) AND  idsocket IN ( select distinct(idsocket) from packets where flags = "RST" and idtrace = %s) )"""

class Processor:
    def __init__(self, cfgfile=gcfgfile):
        self.config = self.loadConfig(cfgfile)
        self.dbObj=database.DbUtil(config=cfgfile)

    def loadConfig(self, configfile):

        config = ConfigParser.ConfigParser()

        try:
            config.readfp(open(configfile))
        except IOError:
            pcaputil.perror( "Couldn't open %s" % configfile)
            sys.exit(2) 
        return config

    def postprocess(self, traceid):
        dbObj = self.dbObj
        dbconn = dbObj.getConn()

        """ and now on to the post processing"""
        pcaputil.pdbg( "done.  now post-processing.  this might take a while")

        """ post-processing -- populating a few more columns using 
            the now complete packet information 
        """
        pcaputil.pdbg( "setting socket durations for trace id: %s" % traceid)
        dbObj.singleItemQuery(dbconn, update_firstpack, (traceid, traceid))
        dbObj.singleItemQuery(dbconn, update_lastpack, (traceid, traceid))
        dbObj.singleItemQuery(dbconn, update_duration, (traceid, traceid))

        pcaputil.pdbg( "setting bytes passed by each socket for trace id: %s" % traceid)
        dbObj.singleItemQuery(dbconn, update_inbytes , (traceid, traceid))
        dbObj.singleItemQuery(dbconn, update_outbytes , (traceid, traceid))

        """
        print "setting socket classifications for trace id: %s" % traceid
        dbObj.singleItemQuery(dbconn, up_flag_full , (traceid, traceid, traceid))
        dbObj.singleItemQuery(dbconn, up_flag_finack , (traceid, traceid))
        dbObj.singleItemQuery(dbconn, up_flag_rst , (traceid, traceid))
        """
        dbObj.returnConn(dbconn)


    def processTraceList(self):
        dbconn = self.dbObj.getConn()

        for idtrace in self.dbObj.queryWithResults(dbconn, select_trace):
            self.postprocess(idtrace[0])
        
        self.dbObj.returnConn(dbconn)

        self.dbObj.cleanup()

if __name__ == "__main__":

    if len(sys.argv) > 2:
        cfgfile= sys.argv[1]
    else:
        cfgfile = gcfgfile

    p = Processor(cfgfile)

    p.processTraceList() 



