#!/usr/bin/env python

"""
Script to extract data from a standard packet capture (tcpdump), write list of unique socket pairs to the database,
and then dumping the packets into logs for subsequent bulk upload

"""
import inc
import os
import sys
import impacket
from impacket.ImpactDecoder import EthDecoder
import pcapy
from pcapy import open_offline
import database
import pcaputil
import ConfigParser

""" code bits borrowed from Impacket/examples/split.py """

cfg_net ="network"
cfg_intnet = "internal_net"
cfg_block_data = "data"
cfg_extractdir = "extractdir"
"""
 simple list of internal networks -- used for determining
 ext ip in 
 
"""
internal_networks = []

writedir="parsed"
gcfgfile = "./conf/properties.cfg"

class FileInfo:
    def __init__(self, filename):
        self.filename = filename
        filesize = -1
        filehash = None
        id = -1

class Decoder:
    def __init__(self, pcapObj, dbObj, finfo, writedir):
        self.decoder = EthDecoder()
        self.dbObj = dbObj
        self.pcap = pcapObj
        self.packetdict = {}
        self.finfo = finfo
        self.outfile = open("%s/%s" % (writedir, self._getOutFile(finfo.filename)), "w")

    def _getOutFile(self, filename):
        ext = "tp"
        f = os.path.basename(filename)
        return "%s.%s" % (f, ext)
    
    def writeFileHeader(self):
        """
            FORMAT:
            
            hash: fdlfdfldjfdlfjdlfjdfljdfldjfld
            filename: /full/file/name
        """
        self.outfile.write("id:%s\n" % self.finfo.id)
        self.outfile.write("hash:%s\n" % self.finfo.filehash)
        self.outfile.write("source:%s\n" % self.finfo.filename)
        self.outfile.write("source size:%s\n" % self.finfo.filesize)

    def start(self):
        # Sniff ad infinitum.
        # PacketHandler shall be invoked by pcap for every packet.

        self.writeFileHeader()

        # throws exception
        # pcapy.PcapError
        while True:
            try:
                self.pcap.loop(0, self.packetHandler)
                break
            except pcapy.PcapError, e:
                pcaputil.perror( e)


    def isInternalNet(self, ip):
        global internal_networks

        for l in internal_networks:
            if not ip.find(l) == -1:
                return True

        return False

    def createSocketEntry(self, inthost, exthost, starttime):

        query = """INSERT INTO sockets (extip, extport, intip, intport, idtrace, starttime)
            VALUES (%s, %s, %s, %s,%s,%s)"""  

        conn = self.dbObj.getConn()
        return self.dbObj.runInsert(conn, query, (exthost[0], exthost[1], inthost[0], inthost[1], self.finfo.id,starttime))
        
    def hosthash (self, inthost, exthost):
        return hash(hash(exthost[0]) ^ hash(exthost[1]) ^ hash(inthost[0]) ^ hash(inthost[1]))

    """
        Logic here -- most packets will just be coming here to get an idsocket for their records

        if it is a new socket, we denote this by first finding a SYN in the packet parse and
        then passing the TIMESTAMP of that syn-packet here.  Otherwise, the starttime will be -1.
    """
    def getSocketUID(self, inthost, exthost, starttime=-1):
        # generate hash

        hosthash = self.hosthash(inthost, exthost)
       
        """ easiest case -- just give me the value from the hash please """
        if self.packetdict.has_key(hosthash):
            if starttime == -1:
                return int(self.packetdict[hosthash])
                """
                IF the socket exists AND we get a timestamp, then we have to replace
                the values for the hosthash key with a new value.
                """
            else:
                idsock = self.createSocketEntry(inthost, exthost, starttime)
                """ replace the value in the hash with the new one """
                self.packetdict[hosthash] = idsock
        else:
            """ 
                we can get here either with a SYN giving us a value for starttime
                or it's possible that the socket conversation was initiated before
                we took the trace in which case we use -1 as the default unknown value
            
            """
            idsock = self.createSocketEntry(inthost, exthost, starttime)
            self.packetdict[hosthash] = idsock

            return idsock


    def packetHandler(self, hdr, data):
        """Handles an incoming pcap packet. This method only knows how
        to recognize TCP/IP connections.
        Be sure that only TCP packets are passed onto this handler (or
        fix the code to ignore the others).

        Setting r"ip proto \tcp" as part of the pcap filter expression
        suffices, and there shouldn't be any problem combining that with
        other expressions.
        """

        # Use the ImpactDecoder to turn the rawpacket into a hierarchy
        # of ImpactPacket instances.
        p = self.decoder.decode(data)

        ip = p.child()
        tcp = ip.child()

        tstamp = hdr.getts()[0]
        src = ip.get_ip_src()
        dst = ip.get_ip_dst()
        sport = tcp.get_th_sport()
        dport = tcp.get_th_dport()

        srchost = (src, sport )
        dsthost = (dst, dport)

        
       

        psize = ip.get_ip_len()
        ttl = ip.get_ip_ttl()

        window_size = tcp.get_th_win()
        rawflags = tcp.get_th_flags()
        flags = pcaputil.getFlagAsString(rawflags)
        seqnum = tcp.get_th_seq()
        acknum = tcp.get_th_ack()

        """ is this a new socket """
        if flags == 'SYN':
            starttime = tstamp
        else:
            starttime = -1

        """ 
            now we need to find the socket GUID
            to use in the packet insert coming up
        """

        if self.isInternalNet(srchost[0]):
            idsock = self.getSocketUID(srchost, dsthost, starttime)
        else:    
            idsock = self.getSocketUID(dsthost, srchost, starttime)
    
        """ so we have a blank first field here to accomodate the no-NULL
            despite auto_increment rule of mysql bulk imports """
        self.outfile.write(""" |%s|%s|%s|%s|%s|%s|%s|%s|%s|%s|%s|%s|%s|%s\n""" % (src, dst, sport, dport, psize, ttl, window_size, rawflags, flags, seqnum, acknum, tstamp, self.finfo.id, idsock))

class Parser:
    def __init__(self, config=gcfgfile):
        self.traceloc = None
        self.extractloc = None
        self.config = self.loadConfig(config)
        self.dbObj=database.DbUtil(config)

    def fileExists(self, finfo):

        """ returns a tuple -- hexdigest and filesize"""
        (md5hash, filesize) = pcaputil.getMd5Hash(finfo.filename)
        filehash = "%32s" % md5hash
        findquery = """SELECT id from tracefile where uid = %s""" 
    
        dbObj = self.dbObj

        conn = dbObj.getConn()
        id = dbObj.singleItemQuery(conn, findquery, (filehash))
        
        if not id is None: # we've seen it
            pcaputil.perror("File exists")
            return True

        finfo.filehash =filehash
        finfo.filesize = filesize

        insquery = """INSERT INTO tracefile (uid, filename, dtcreated, size, flags) VALUES (%s, %s, NOW(), %s, 1)""" 

        traceid = dbObj.runInsert(conn, insquery, (finfo.filehash, finfo.filename, finfo.filesize))

        finfo.id = traceid

        dbObj.returnConn(conn)

        return False

    def loadConfig(self, configfile):
        global internal_networks
        global writedir
        
        config = ConfigParser.ConfigParser()

        try:
            config.readfp(open(configfile))
        except IOError:
            pcaputil.perror("Couldn't open %s" % configfile)
            sys.exit(2) 

        try:

            if config.has_section(cfg_block_data):
                try:
                    self.traceloc = config.get('data', 'tracedir')        
                    if len(self.traceloc) == 0:
                        pcaputil.perror( "no trace location set")

                    self.extractloc = config.get('data', 'extractdir')        
                    if len(self.extractloc) == 0:
                        pcaputil.perror( "no extract location set")

                except ConfigParser.NoOptionError:
                    pcapUtil.perror( "Illegal configuration -- network block")
                    sys.exit(1)
            else:
                pcaputil.perror("No data dir override found")

        except ConfigParser.NoOptionError:
            pcaputil.perror( "No data dir override found")

        if config.has_section(cfg_net):
            # return all items in section
            try:
                intnetlist=config.get(cfg_net, cfg_intnet)
                if len(intnetlist) == 0:
                    pcaputil.perror("illegal list")
                    sys.exit(1)

                l = intnetlist.split(',')

                for item in l:
                    internal_networks.append(item.strip())

            except ConfigParser.NoOptionError:
                pcaputil.perror("Illegal configuration -- network block")
                sys.exit(1)

        else:
            pcaputil.perror("Illegal configuration\n")
            sys.exit(1)

        return config

    def processFile(self, finfo):
        dbconn = self.dbObj.getConn()
        """ this does dual work -- it adds file if it doesn't exist
            and the obvious checks if it is there """
        if self.fileExists(finfo):
            print "We've already seen this file"
            return

        p = open_offline(finfo.filename)
        p.setfilter(r'ip proto \tcp')
        Decoder(p, self.dbObj, finfo, self.extractloc).start()

    def process(self):
        traceloc = self.traceloc

        """ crack open a directory and handle each file """
        if os.path.isfile(traceloc):
            pcaputil.pdbg("processing %s" % traceloc )
            finfo = FileInfo(traceloc)
            self.processFile(finfo)
        else:
            for file in os.listdir(traceloc):
                path = os.path.join(traceloc, file)
                if os.path.isfile(path):
                    pcaputil.pdbg( "processing %s" % path)
                    finfo = FileInfo(path)
                    self.processFile(finfo)
                else:
                    """ skip for now -- else we get all recursive """
                    pcaputil.pdbg( "skipping subdir %s" % path)

        self.dbObj.cleanup()
        

if __name__ == "__main__":

    if len(sys.argv) < 2:
        print "usage: %s <configfile>" % sys.argv[0]
        sys.exit(1)
    else:
        cfgfile = sys.argv[1]


    parser = Parser(cfgfile)
    parser.process()


