#!/usr/bin/env python

import MySQLdb
from MySQLdb import Error
import ConfigParser
import sys


#############################################
DB="db"
dbUserKey="user"
dbPassKey="pass"
dbDbKey="database"
dbHostKey="dbhost"

def perror(str):
    sys.stderr.write("%s\n" % str)

class DbUtil:

    def __init__(self, config="./properties.cfg"):
        self.config = ConfigParser.ConfigParser()
        self.conn = None

        try:
            self.config.readfp(open(config))
        except IOError:
            perror("Couldn't open %s" % config)
            sys.exit(2)

        if self.config.has_section(DB):
            # return all items in section
            try:
                self.mdbuser=self.config.get(DB, dbUserKey)
                if len(self.mdbuser) == 0:
                    perror("bad user")
                    sys.exit(1)

                self.mpassword=self.config.get(DB, dbPassKey)
                if len(self.mpassword)== 0:
                    perror("bad pass")
                    sys.exit(1)

                self.mdb=self.config.get(DB, dbDbKey)
                if len(self.mdb)== 0:
                    perror("bad db")
                    sys.exit(1)

                self.mhost=self.config.get(DB, dbHostKey)
                if len(self.mhost)== 0:
                    self.mhost = "localhost"


            except ConfigParser.NoOptionError:
                perror("Illegal configuration -- requires values for user, pass, and tns")
                sys.exit(1)

        else:
            perror("Illegal configuration\n")
            sys.exit(1)


    def runInsert(self, conn, sql, paramlist):

        id = -1
        try:
            cursor = conn.cursor()
            cursor.execute(sql, paramlist)

            id = cursor.lastrowid

            cursor.close()
            """
            cursor = conn.cursor()
            cursor.execute("SELECT LAST_INSERT_ID()")
            id = "%s" % cursor.fetchone()
            cursor.close()
            #print id
            """

        except MySQLdb.Error, e:
            perror("Error %d: %s" % (e.args[0], e.args[1]))
            sys.exit(1)

        return id

    """ usage: 

            for result in dbobj.queryWithResults(conn, sql) 

    """
    def queryWithResults(self, conn, sql, paramlist=(), fetchsize=500):

        try:
            cursor = conn.cursor()
            cursor.execute(sql, paramlist)
            while True:
                results = cursor.fetchmany(fetchsize)
                if not results: break
                for result in results:
                    yield result

            cursor.close()
            conn.commit();
        except MySQLdb.Error, e:
            perror("Error %d: %s" % (e.args[0], e.args[1]))
            sys.exit (1)

    def singleItemQuery(self, conn, sql, paramlist=()):
        id = -1
        try:
            cursor = conn.cursor()
            cursor.execute(sql, paramlist)
            id = cursor.fetchone()
            cursor.close()
            conn.commit();
        except MySQLdb.Error, e:
            perror("Error %d: %s\nsql: %s" % (e.args[0], e.args[1], sql))
            sys.exit (1)

        return id

    def getConn(self, dbuser=None, dbpass=None, dbase=None, dbhost=None):
        """
            we can hide all kinds of stuff here -- conn pool
            single shared connection in a single thread model
            whatever we need
        """
        if self.conn is None:
            if dbuser is None:
                dbuser=self.mdbuser
            if dbpass is None:
                dbpass=self.mpassword 
            if dbase is None:
                dbase=self.mdb
            if dbhost is None:
                dbhost=self.mhost

            conn = None
            try:
                #print "Connection for %s@%s - %s" % (mhost, dbuser, dbase)
                conn = MySQLdb.connect (host = dbhost,
                               user = dbuser,
                               passwd = dbpass,
                               db = dbase)

            except MySQLdb.Error, e:
                perror("Get Conn Error %d: %s" % (e.args[0], e.args[1]))
                sys.exit (1)
        else:
            conn = self.conn

        return conn


    def returnConn(self,conn):

        if conn != self.conn:
            try:
                if not conn == None:
                    conn.close()

            except MySQLdb.Error, e:
                perror("Close Conn Error %d: %s" % (e.args[0], e.args[1]))
        
        else:   
            self.conn.commit()

    def cleanup(self):
        if not self.conn == None:
            self.conn.close()

if __name__ == "__main__":
    
    dbutil = DbUtil("/Users/uzada/Dev/packetdb/conf/properties.cfg")
    conn = dbutil.getConn()
    """
    query = "select id, str from test"
    for rs  in dbutil.queryWithResults(conn, query):
        print rs
    """
    dbutil.returnConn(conn)
