--

--
-- flags field has the following meaning
--      1 - entry created via extract process
--      2 - dump loaded successfully from file
--
CREATE TABLE tracefile (
    id int auto_increment,
    uid varchar(32),
    filename varchar(100),
    dtcreated date,
    dtloaded date,
    size int,
    flags int,
    PRIMARY KEY(id)
);

--
-- dataflag 
---     0 - start or end of socket not in db -- no SYN, FIN,ACK, or RST
---     1 - complete conversation SYN + FIN,ACK OR RST in db
---     2 - FIN,ACK only, no SYN
---     4 - RST only, no SYN
--
CREATE TABLE sockets
(
    idsock int auto_increment,
    extip varchar(15),
    extport int,
    intip varchar(15),
    intport int,
    idtrace int,
    inbytes int,
    outbytes int,
    duration int,
    dataflag int default 0,
    starttime int,
    firstpacket int,
    lastpacket int,
    FOREIGN KEY (idtrace) REFERENCES tracefile(id)
                ON DELETE CASCADE,
    PRIMARY KEY(idsock)
);


CREATE TABLE packets (
    idpack int auto_increment,
    srcip varchar(15),
    dstip varchar(15),
    srcport int,
    dstport int,
    packetsize int,
    ttl int,
    winsz int,
    rawflags int,
    flags varchar(7),
    seqnum int,
    acknum int,
    timestamp int,
    idtrace int,        
    idsocket int,        
    FOREIGN KEY (idtrace) REFERENCES tracefile(id)
                ON DELETE CASCADE,
    FOREIGN KEY (idsocket) REFERENCES sockets(idsock)
                ON DELETE CASCADE,
    PRIMARY KEY(idpack)
);


create table external_ip 
( 
    idip int auto_increment,
    ip varchar(15),
    PRIMARY KEY(idip)
);


create index ix_ip_port_packets on packets(srcip, dstip, srcport, dstport);
create index ix_flags_sockets on packets(idsocket, flags, idtrace);
create index ix_ext_ip_ip on external_ip(ip);



